require "consul/client/version"
require "consul/client/config"
require "consul/client/connection"
require "consul/client/offline_connection"
require "consul/client/fake_http_response"
require "consul/http"
require "consul/response"
require "consul/data"

require "forwardable"

module Consul
  module Client

    class << self
      extend Forwardable

      def_delegators :connection, *(Connection.instance_methods - Object.instance_methods)

      def new
        if is_online?
          Connection.new
        else
          OfflineConnection.new
        end
      end

      def connection
        @connection ||= new
      end

      def is_online?
        if defined?(Chef)
          !::Chef::Config[:solo]
        else
          false
        end
      end

      def reconnect
        @connection = new
      end

    end

  end
end
