require "psych"
require "fileutils"

module Consul::Client
  class Connection

    def config
      @config ||= Config.new
    end

    def read_obj(keypath, opts={})
      keyname = File.basename(keypath)
      response = http.get("/kv" + path(keypath) + (opts[:recursive] ? '?recurse' : ''))
      if response.length == 1 && response.keys.first == keyname
        response[keyname]
      else
        response
      end
    end

    def read(keypath, opts={})
      response = read_obj(keypath, opts)
      read_value_r(response)
    end

    def read_value_r(response)
      if response.respond_to?(:keys)
        Hash[ response.map{ |k,v| [k,read_value_r(v)] } ]
      else
        response.value
      end
    end

    def delete(keypath, opts={})
      http.delete("/kv" + path(keypath) + (opts[:recursive] ? '?recurse' : '')).value
    end

    def write(keypath,data=nil,path_mimic=false)
      data = encode(data)
      http.put("/kv" + path(keypath), data)
    end

    #def lock(keypath, data=nil)
    #  write(keypath + "?acquire=#{session_id}", data)
    #end

    #def unlock(keypath, data=nil)
    #  write(keypath + "?release=#{session_id}", data)
    #end

    #def session_id
    #  @@session_id ||= http.put("/session/create")
    #end

    #def has_lock?(keypath)
    #  read("/kv/" + keypath).session == session_id
    #end

    def http
      @http ||= Consul::Http.new
    end

    def path(keypath)
      File.join("/" + config.namespace,  keypath).gsub(%r{//}, '/')
    end

    def decode_r(data)
      if data.respond_to?(:keys)
        Hash[ data.map{ |k,v| [k,decode_r(v)] } ]
      else
        decode(data.value)
      end
    end

    def decode(value)
      Psych.safe_load(value, [Symbol])
    end

    def encode(value)
      encoded = Psych.dump(value)
      if decode(encoded) != value
        raise TypeError, "Unable to relaibly decode #{value.class}"
      end
      encoded
    end

    def encode_r(data)
      if data.respond_to?(:keys)
        Hash[ data.map{ |k,v| [k, encode_r(v)] } ]
      else
        encode(data)
      end
    end
  end
end
