require 'base64'
require 'forwardable'

module Consul
  class Data
    extend Forwardable

    def_delegators :value, :to_a, :to_h, :to_i, :to_f, :to_s

    def initialize(response_hash)
      @response = response_hash
    end

    def value
      text = Base64.decode64(hash_map('Value'))
      try_decode(text)
    end

    def flags
      hash_map(__callee__)
    end
    [:key, :lock_index, :create_index, :modify_index, :session].each{ |a| alias_method a, :flags }

    private
    def hash_map(key)
      @response[key.to_s.split('_').map{ |w| w.capitalize }.join]
    end

    def try_decode(text)
      begin
        decoded = Consul::Client.decode(text)
      rescue Psych::DisallowedClass, Psych::SyntaxError
      end
      decoded || text
    end

  end
end
