require 'psych'
require 'net/http'
require 'forwardable'

module Consul
  class Response
    extend Forwardable
    include Enumerable

    attr_reader :response, :data
    def_delegators :response, :body, :code, :code_type
    def_delegators :data, :keys, :values, :each, :to_a, :to_h, :length, :map, :[]

    class KeyError < RuntimeError; end

    def initialize(net_http_response, request_path='/')
      @response = net_http_response
      @request_path = File.dirname(request_path)
    end

    def data
      @data ||= init_data
    end

    def value
      @value ||= to_ruby
    end

    #def locked?
    #  to_h.has_key?("Session")
    #end

    def success?
      response.code_type == Net::HTTPOK
    end

    private

    def init_data
      {}.tap do |retval|
        to_ruby.each do |metadata|
          hash_path = metadata['Key'].gsub(%r{^/?#{@request_path}/?}, '')
          retval.merge!(build_data_hash(hash_path,metadata), &merger)
        end
      end
    end

    def build_data_hash(hash_path, consul_metadata)
      hash_path.split('/').reverse.inject(Data.new(consul_metadata)) { |memo,key| { key => memo } }
    end

    def to_ruby
      if response.code_type == Net::HTTPOK
        Psych.safe_load(response.body)
      elsif response.code_type == Net::HTTPNotFound
        raise KeyError, "Requested key not found"
      else
        raise "Request failed"
      end
    end

    def merger
      proc { |key,new,old| new.respond_to?(:merge) && new.class == old.class ? new.merge(old, &merger) : old }
    end
  end
end
